/**
 * @jest-environment jsdom
 */

// Copyright (c) 2023 Tudor Stanciu

import axios from "axios";
import { TuitioClient } from "../client";
import { fetch } from "../state";

jest.mock("axios");

const mockResult = (expiresIn: number = 240) => ({
  result: {
    token: "token-mock",
    expiresIn
  },
  error: null
});

test.each([240, 100, 20, 10, 5])("Tuitio client: successfully account login", async (expiresIn: number) => {
  const mock = mockResult(expiresIn);

  (axios.request as jest.Mock).mockResolvedValue({
    data: {
      ...mock
    }
  });

  const client = new TuitioClient("https://test.com/api");
  const loginResult = await client.login("user", "pass");

  expect(loginResult.result?.token).toBe(mock.result.token);
  expect(loginResult.result?.expiresIn).toBe(mock.result.expiresIn);
  expect(loginResult.result?.validUntil).toBeDefined();
  const validUntilTime = loginResult.result?.validUntil.getTime();
  if (validUntilTime) {
    const now = new Date();
    expect(validUntilTime).toBeGreaterThan(now.getTime());
  }
  expect(loginResult.error).toBeNull();

  const storage = fetch();
  expect(storage).toBeDefined();
  expect(storage?.token).toBe(mock.result.token);
  expect(storage?.userName).toBe("user");
});

test("Tuitio client failed authentication", async () => {
  (axios.request as jest.Mock).mockRejectedValueOnce({
    response: {
      status: 500,
      data: { title: "Internal server error" }
    }
  });

  const client = new TuitioClient("https://test.com/api");

  try {
    await client.login("user", "pass");
  } catch (error: any) {
    expect(error.title).toBe("Internal server error");
    expect(error.message).toBe("Internal server error");
  }
});

test("Tuitio client failed authentication with unhandled exception", async () => {
  (axios.request as jest.Mock).mockRejectedValueOnce({
    response: {
      status: 500,
      error: { message: "Internal server error" }
    }
  });

  const client = new TuitioClient("https://test.com/api");

  try {
    await client.login("user", "pass");
  } catch (error: any) {
    expect(error.response.status).toBe(500);
    expect(error.response.error.message).toBe("Internal server error");
  }
});
